//Author: BT

//Store the ID of the track that was received from the previous call to the server.
//This allows us to only change the data on the page if the track ID has changed.
var previousTrackId  = -1;
var previousMutedVol =  0;

function sendPlayCommand(trackNo)
{
	//console.log("Sending command: play track ID " + trackNo);
	setControlIconToPause();
	$.post("playTrack",{"id":trackNo});
}

function sendVolumeCommand(volume)
{
	//console.log("Sending command: change volume to " + volume);
	$.post("setVolume",{"vol":volume});
}

function sendPreviousTrackCommand()
{
	//console.log("Sending command: previous track");
	$.post("previousTrack");	
}

function sendNextTrackCommand()
{
	//console.log("Sending command: next track");
	$.post("nextTrack");	
}

function sendTogglePauseCommand()
{
	//console.log("Sending command: togglePause");
	$.post("togglePause");	
}

function sendStopTrackCommand()
{
	//console.log("Sending command: stop track");
	$.post("stopPlayback");	
}

function populateNowPlaying(track)
{
	var currentTrack = getTrackById(track["trackID"]);
	//console.log(currentTrack);
	
	//Some things need to be updated regardless of whether the track changed.
	//This includes the timer and whether or not the track is paused.
	if(currentTrack.length != undefined)
		updateProgress(track["currentPosition"],currentTrack.length);
	
	updateVolume(track["volume"]);
	
	// //console.log("stopped: " + track["stopped"]);
	// //console.log("paused : " + track["paused"]);
	// //console.log("");
	
	if(!track["stopped"])
	{
		unsetStoppedState();
		updatePaused(track["paused"]);
	}
	else
	{
		setStoppedState(track["trackID"],track["paused"]);
	}
	
	//timesIndexed will be -1 if we've only done the first indexing.
	//Since we have no idea when the client will join and the timesIndexed
	//could be any arbitrary number, we'll sync our value with the server
	//for the first time only.
	if(timesIndexed == -1)
	{
		timesIndexed = track["timesIndexed"];
	}
	//Reload the track index if the index counts don't match.
	else if(track["timesIndexed"] != timesIndexed)
	{
		//console.log("timesIndexed = " + track["timesIndexed"] + "; I have " + timesIndexed);
		loadTrackIndex(false);	
		timesIndexed = track["timesIndexed"];
		$(".tab1").click();
	}
	
	//Populate content area with art, header, and bio if it changes
	if(previousTrackId != track["trackID"])
	{
		previousTrackId = track["trackID"];
		fillCurrentTrackHeader(currentTrack.artist, currentTrack.title, currentTrack.album, currentTrack.year, currentTrack.genre);
		setAlbumArt(currentTrack.artist, currentTrack.album);
		setArtistBio(currentTrack.artist);
		$('.footercontrols_bio').click();
	}
}

function updateVolume(serverVolumeValue)
{
	if(serverVolumeValue != serverPreviousVolume || currentVolume != serverVolumeValue)
	{
		serverPreviousVolume = serverVolumeValue;
		$('.volumeSlider').simpleSlider("setValue", serverVolumeValue/100);
		setVolumeThumb(serverVolumeValue);
	}
}

function unsetStoppedState()
{
	setPauseToggleEvent();
}

function setStoppedState(trackID, isPaused)
{
	//If we're playing, the Play/Pause icon will be a pause icon.
	//Now that we're stopping, we'll need to change it to the play icon
	if(isPaused == 0) 
	{
		setControlIconToPlay();
	}
	
	$('img.playpause').unbind().click(function(){
		sendPlayCommand(trackID);
		setControlIconToPause();
	});
	
	$("img.playpause").hover(
		function(){
			$(this).animate({opacity:1},100);
		},
		function(){
			$(this).animate({opacity:0.5},100);
		}
	);		
}

//Updates the pause button source depending on whether the track is currently playing.
//This is dependent only on the server's response. Therefore, without any additional
//handling, a user clicking the play/pause button will not receive visual feedback for
//at least 1 second (the refresh rate). The instant changing will be handled with an onclick
//handler.
function updatePaused(isPaused)
{	
	if(isPaused == 0)
	{
		setControlIconToPause();	
	}
	else
	{
		setControlIconToPlay();
	}
}

//Takes a given artist's name and populates the bio section with information about
//that artist using the lastFM API. We trust that lastFM will return data and that
//the artist is not too obscure.
function setArtistBio(artist)
{
	//Set bio information to "Loading" while lastFM is queried.
	$("div.nowplaying_biotext p").empty().append("Loading...");
	
	//Set similar artist information to "Loading" while lastFM is queried.
	$("div.nowplaying_similarartists").empty().append("<br />Loading...");	
	
	getArtistInfo(artist, function(data){
		if (navigator.userAgent.indexOf("Firefox")!= -1)
		{
			data = JSON.parse(data);
		}
		
		var bioString;
		
		if(data.message) //This child will exist if there was an error getting the artist from the API
		{
			bioString = "Not available.";
		}
		else
		{
			bioString = data.artist.bio.summary.replace(/<a(\s[^>]*)?>(.*?)<\/a>/ig, '$2');
			bioString = bioString.replace(/\(\.\.\.\)/g, '');
			var lastFMPlug = bioString.indexOf("Read more about");
			bioString = bioString.substr(0,lastFMPlug);
		}

		$("div.nowplaying_biotext p").empty().append(bioString);
		
		var similarArtistMarkup = "<br />";	
		
		if(data.message) //This child will exist if there was an error getting the artist from the API
		{
			similarArtistMarkup += "Not available.";
		}
		else
		{
		
			var similarArtists = data.artist.similar.artist;
			if(similarArtists.length != 0)
			{
				similarArtists.forEach(function(element){
					similarArtistMarkup += "<span class='similarArtist'>" + element.name + "</span><br />";
				});
			}
		}
		
		$("div.nowplaying_similarartists").empty().append(similarArtistMarkup);
	});
}

//Sets the album cover source in the now playing section based on the artist and album.
function setAlbumArt(artist, album)
{
	//Set album art to blank until it has been loaded
	$("img.nowPlayingCover").attr("src", "placeholder.png");
	
	getAlbumCover(artist, album, function(data){
		var albumCover = getLargeAlbumUrl(data);
		if(albumCover == "")
		{
			albumCover = "placeholder.png";
		}
		$("img.nowPlayingCover").attr("src", albumCover);
	});
}

//Fills the header for the now playing section with the track metadata of the currently
//playing track.
function fillCurrentTrackHeader(artist, title, album, year, genre)
{
	var markup = "<span class='nowplaying_artist nowplaying_text'>" + artist + "</span> " +
		"<span class='nowplaying_title nowplaying_text'>"  + title  + "</span><br />" +
		"<span class='nowplaying_album nowplaying_text'>"  + album  + "</span>";
	if(year != "(null)")
	{
		markup += "<span class='nowplaying_year nowplaying_text'> (" + year   + ")</span>";
	}
	if(genre != "(null)")
	{
		markup += "<br /><span class='nowplaying_genre nowplaying_text'>"  + genre  + "</span>";
	}
	$(".nowplaying_header").empty().append(markup);
	
	$(".nowplaying_artist").unbind().click(function(){
		$(".tab1").click();
		populateArtistAlbums(contentArea,artist,getArtistAlbums(artist));
		configureNowPlayingGoBack(function(){$(".tab3").click();});
	});
	
	$(".nowplaying_album").unbind().click(function(){
		$(".tab1").click();
		loadAlbumData(contentArea,artist,getAlbumObject(artist, album).tracks);
		configureNowPlayingGoBack(function(){$(".tab3").click();});
	});	
}

//Sets the back button action for actions clicked via the now playing screen.
//This is slightly different than the regular go back function since it doesn't
//clear the content area before calling the callback function.
function configureNowPlayingGoBack(callback)
{
	$("#back").unbind();
	$("#back").click(function() {
		callback();
	});
}